% conex.m
%  Hansen consumption example
%  Forms innovations.

%  Notes, Nov 15, 1993.  We
%  have added a crude way of estimating the
%  initial state, which helps a lot. 
%  We also create the backwards "Skoog" system,
%  and create the Skoog innovations.
%  To be added: a truly backwards K-filter to
%  estimate initial condition.

%  NOTE: the proximity of the rho's to 1 dictates the convergence
%  properties of the Kalman filter A - KG.  We probably
%  want to restrict these roots to stay pretty far from 1 in order
%  to do away with initial condition problems.


 clexcon
% c2=c2*.01;
solvea

sy=ss;

z1=aimpulse(ao,c,ss,2,20);
%plot(z1),title('response of services to endow. innov.'),pause
z2=aimpulse(ao,c,sc,2,20);
%plot(z2),title('response of cons to endow. innov.'),pause

% get canonical representation

[lambdah,pihh,ubh]=can(beta,lambda,pih,deltah,thetah,ub,a22)


%lambdah=lambda; pihh=pih;   % temporary fix, hope that repres is canonical!
%  get the * matrices for the inverse, or *, system
%  Note that we let *s denote the appropriate inverse system matrix

lambdas=-pihh\lambdah;
pihs=inv(pihh);
deltas=deltah-(thetah/pihh)*lambda;
thetas=thetah/pihh;




%  Compute Ms

Ms=(pihs' +beta*(thetas'\(eye(nc)-beta*deltas'))*lambdas');
Ms=Ms*(phic(1,:)');


%  Now form the nonstochastic steady state values
%  of c.

GG=(lambdas*((eye(nb)-deltas)\thetas)+pihs)*Ms;
%
%  Now compute Psi, the matrix of co-integrating
%  vectors
%  recall that HH=null(GG') means that
%  GG'*HH =0 or HH'*GG=0; this implies
%  that Psi =HH'   .

HH=null(GG');
Psi=HH';


%

%  compute innovations
%
%rho=[.99 .99 .99];
rho=[.0 .0 .0];
%rho=[.01 .05 .1];

Gam= diag(rho);   % This is the serial correlation of errors

R=eye(3)*.005;   %  Measurement error variances

%nnc=5;
%steadst;

x0=[ 20 20 20 200 1 0 0]';

t1=250;
sy=[sc];                   %  Set observer equal to consumption vector

rand('seed',.5533)

asimul
rand('normal')
uu=rand(t1,3);
G11=chol(R);

meas=ltitr(Gam,eye(3),uu*G11);
yt=y;  %  true y

y=y+meas


x0=[ 2 20 2 100 1 0 0]';  %  reset initial condition for K-filter

T=length(y);
dy=y(2:T,:)-y(1:T-1,:)*Gam;
T=T-1;


Omega=R     ;  %measurement error covariance matrix
Sigma = c*c';

 Sigma=Sigma+.00001*eye(7);   %  Trick to make Sigma full rank

D=sy;       % observer matrix
%Dp  = D';
Dbar=D*ao-Gam*D;
Obar=Omega+D*Sigma*D';
[K,S]=kfilter(ao,Dbar,Sigma,Obar,Sigma*D');
Sigmaz=Obar+Dbar*S*Dbar';
Sigmazi=inv(Sigmaz);

%  Compute phoney initial condition
%  The procedure will be to solve the steady state equation
%  (I - (A-KG)) \hat x = D \hat y 
%   for the first four \hat x's as functions of the exogenous
%   x's and the first y observation.


AKG=ao-K*Dbar;
temp=eye(7)-AKG;
x00=(temp\K)*dy(1,:)';

dyini=mean(dy(1:5,:));

A11=temp(1:4,1:4); A12=temp(1:4,5:7);
x10=A11\(-A12*[1 0 0]' +K(1:4,:)*dyini');
x0=[x10' 1 0 0]';


%  Construct "smoother"  using Anderson-Moore formulas
 
endd=Dbar'/(Dbar*S*Dbar' + Omega);

sig1=S*AKG';
sig2=S*(AKG'^2);
sig3=S*(AKG'^3);
sig4=S*(AKG'^4);
sig5=S*(AKG'^5);
sig6=S*(AKG'^6);


k2=sig1*endd;
k3=sig2*endd;
k4=sig3*endd;
k5=sig4*endd;
k6=sig5*endd;
k7=sig6*endd;

%  Compute backwards Skoog filter to get the initial condition

%xb=zeros(T,7);
%ennov=zeros(T,nc);
%xb(T,:)=x0';
%ennov(T,:)=dy(T,:)-x0'*K ;
%for i=T-1:-1:1;
%  xb(i,:)=xb(i+1,:)*ao+ennov(i+1,:)*Dbar;
%  ennov(i,:)=dy(i,:)-xb(i,:)*K;
%end;


%x0=xb(1,:)';  % shut down the backward Kalman filter as device for
               % initial conditions because the state is not
               % interpretable as comparable with original x


xt(1,:)=x0';
innov(1,:)=dy(1,:)-x0'*Dbar';
for i=2:T;
  xt(i,:)=xt(i-1,:)*ao'+innov(i-1,:)*K';
  innov(i,:)=dy(i,:)-xt(i,:)*Dbar';
end;


x000=x0+ k2*innov(1,:)' + k3*innov(2,:)' +k4*innov(3,:)'+k5*innov(4,:)'...
       + k6*innov(5,:)' + k7*innov(6,:)';

return


%  Now compute the altered innovations
%  
Psi1=inv([ 1 zeros(1,nc-1);Psi])
psi1=Psi1(:,1);    




return

%  Fix the FIRST ennov(1,:)
rnnov(1,:)=zeros(1,3);
xt(1,:)=x0';
rnnov(2,:)=dy(2,:)-x0'*Dbar' + psi1'*(- dy(1,1)+ x0'*(Dbar(1,:)'));
for i=3:T;
  xt(i,:)=xt(i-1,:)*ao'+rnnov(i-1,:)*K';
  rnnov(i,:)=dy(i,:)-xt(i,:)*Dbar' + psi1'*(rnnov(i-1,1) - dy(i-1,1) ...
          + xt(i-1,:)*(Dbar(1,:)'));

end;


