%negishi2.m
%   Computes the canonical mongrel service technology for
%   parameter alpha (the Pareto weight on the first consumer)
%
%   The mongrel technology is
%
%       H(t) = Deltah H(t-1) + Thetah c(t)
%
%       s(t) = Lambda H(t-1) + Pih c(t)
%
%   The mongrel technology shock is given by the series connection
%   of the three state space systems (A1,B1,C1,D1), (AA,BB,GG,HH),
%   (Deltah,Thetah,Lambdah,Pih).  We calculate
%   a minimal realization of the resulting system for the
%   mongrel shock, and put it in the system (Am,Bm,Cm,Dm).
%   The mongrel shock is thus described by
%
%        Z(t+1) = Am Z(t) + Bm w(t+1)
%
%         bb(t) = Cm Z(t) + Dm w(t+1)
%
%   In using this program, it is important to set the initial
%   condition for the state appropriately.  The given initial
%   conditions for h01 and h02 are loaded ino the SHOCK process,
%   and the initial conditions for the MONGREL h01,h02 are set
%   to zero.
%   See check2.m for a check on this program.




[lambdah1,pihh1]=canonpr(beta,lambda1,pih1,deltah1,thetah1);
[lambdah2,pihh2]=canonpr(beta,lambda2,pih2,deltah2,thetah2);

[ns1,nh1]=size(lambdah1);
[ns2,nh2]=size(lambdah2);

%AA=[deltah1 - (thetah1/pihh1)*lambdah1, zeros(nh1,nh2);...
%    zeros(nh2,nh1), deltah2 - (thetah2/pihh2)*lambdah2]


BB=[thetah1/pihh1, zeros(ns1,ns2);...
   zeros(ns2,ns1), thetah2/pihh2];

GG=[-(sqrt(alpha)*pihh1)\lambdah1, -(sqrt(1-alpha)*pihh2)\lambdah2];

HH=[ inv(sqrt(alpha)*pihh1), inv(sqrt(1-alpha)*pihh2)];

AA=[deltah1-(thetah1/pihh1)*lambdah1,zeros(nh1,nh2);zeros(nh2,nh1),...
      deltah2-(thetah2/pihh2)*lambdah2];


[K,S]=kfilter(AA,GG,BB*BB',HH*HH',BB*HH')

Omega=GG*S*GG'+HH*HH';

r=chol(Omega);

rpih=inv(r');

Deltah=AA-K*GG;
Thetah=K;
Pih=rpih;
Lambdah=-rpih*GG;

[nz,nw]=size(c2);

A1=[a22 zeros(nz, nh1+nh2); zeros(nh1,nz) deltah1 zeros(nh1,nh2);...
      zeros(nh2, nz+nh1) deltah2];

B1=[c2; zeros(nh1+nh2,nw)]; %

Ct1=[ub1 zeros(ns1,nh1 +nh2); ub2 zeros(ns2,nh1+nh2);...
   zeros(ns1,nz) lambda1 zeros(ns1,nh2);zeros(ns2,nz+nh1) lambda2];

Dt1=zeros(ns1+ns2+ns1+ns2,nw);

mul=[eye(ns1) zeros(ns2) - eye(ns1) zeros(ns2);...
   zeros(ns1) eye(ns2) zeros(ns1) -eye(ns2)];

mul=mul  ; 
C1=mul*Ct1;
D1=mul*Dt1;



GG2=[-(pihh1)\lambdah1, -(pihh2)\lambdah2];

HH2=[ inv(pihh1), inv(pihh2)];

% now use series to stack and merge the three systems

[A3,B3,C3,D3]=series(A1,B1,C1,D1,AA,BB,GG2,HH2);
[A4,B4,C4,D4]=series(A3,B3,C3,D3,Deltah,Thetah,Lambdah,Pih);

%[Am,Bm,Cm,Dm]=minreal(A4,B4,C4,D4)
