function [f,p] = policyi(A,B,R,Q,S)
%function [f,p] = policyi(A,B,R,Q,S)
%  Policy iteration algorithm of Howard applied to linear regulator
%  Also known as Newton's method      
%
%     POLICYI calculates f of the feedback law:
%               
%		u = -fx
%  
%  that maximizes the function:
%
%          sum {beta^t [x'Qx + u'Ru +2x'Wu] }
%  
%  subject to 
%		x[t+1] = Ax[t] + Bu[t] 
%
%  where x is the nx1 vector of states, u is the kx1 vector of controls,
%  A is nxn, B is nxk, Q is nxn, R is kxk, W is nxk.
%                
%    Also returned is p, the steady-state solution to the associated 
%  discrete matrix Riccati equation.
%
[n1,n2]=size(A);
[m1,m2]=size(B);
K1=zeros(m2,n1);
tol=.00001;



  dd=1;
  it=1;
  maxit=1000;
  % check tolerance; for greater accuracy set it to 1e-10
  while (dd>1e-6 & it<=maxit);
    K=K1;
    C=R+K'*S+S'*K+K'*Q*K;
    G=A+B*K;
    X=-dlyap(G,C);
    K1=-(Q+B'*X*B)\(S+B'*X*A);

    dd=max(max(abs(K1-K)));
    it=it+1;

  end;
f=-K1;p=X;
return

