
%seasadj.m  stripped down version of fildemo.m
%echo on
%     This program uses clcssical methods to solve a univariate 
%     signal extraction problem.  References for the methods 
%     under discussion are Chapter XI of Sargent's Macroeconomic
%     Theory (e.g., see exercise 2 and the section on
%     "the optimal filtering formula"); Peter Whittle's book
%     PREDICTION AND REGULATION BY LINEAR LEAST SQUARES METHODS,
%     (U. of Minnesota Press, 1983); and the article "Seasonality
%     in Regression," by Sims in JASA in 1974.
     %press a key to continue
clc
%     The program prompts the user for two autoregressive polynomials
%     and two moving average polynomials, one of each for
%     for a "signal", the other for a seasonal "noise".
%     The problem is to construct the linear least squares estimator
%     of the "signal" at time t as function of current and past (in
%     the case of the one-sided filter) and also future (in the case
%     of the two-sided filter) observations on the signal plus noise.
%    
%     The a.r. polynomials must be input in the form
%
%       [a(1) a(2) . . . a(r)]
%
%     where a(j) is the coefficient on lag j.
%     Similarly, the m.a. polynomials are of the form
%     [b(0) b(1) . . . b(m)]  where b(j) is the coefficient on
%     the white noise at lag j.
%     The program obtains both two-sided and one-sided filters.
      %press any key to continue demonstration
clc
%     The two-sided filter is formed using the clcssical formula
%
%       h2(z)= Sx(z)/(Sx(z)+Su(z))
%
%     where Sx(z) is the spectral density of the signal and
%     where Su(z) is the spectral density of the noise.
%
     %press any key to continue the demonstration
%     You will now be prompted for various parameter values.
%     The first time you use the program we suggest the following
%     settings:
% 
%     n=100 or 200     (number of ordinates in Fourier transform)
%     a1=.8
%     b1=1
%     a2=[0 0 0 .7]    (moving average polynomial for noise)
%     b2=1             (moving average polynomial for noise)
%     pick noise variances as you wish.
%
%     NOTE:  The results get more accurate when larger n is chosen
%     The recommended number of frequency points n is 100 or 200.
%  n   %  n must be in storage     % number of frequency points n  
clc     %  set n at same value as in foreseas
%     The recommended setting is a1=.8
a1= [1.2 -.3]                        %  give a.r. for "signal" ')
a1=[1 -a1];

%     The recommended setting is b1=1
b1=1      % give m.a. polynomial for "signal" ')
se1=1       % innov variance of signal' )

%     The recommended setting is a2=[0 0 0 .8]
a2=[0 0 0 .8 0 0 0 .1]         % a.r. polynomial for noise ')
a2=[1 -a2];
clc
%     The recommended setting is b2=1
b2=1      %  nput('give m.a. polynomial for noise ')
se2=.5      %input('give innovn variance of noise ')
clc
%     Now we'll form the spectral density of the signal
%     and store it in ss1 and the spectral density of the noise
%     and store it in ss2.  First we form the frequency responses
%     of the filters b1/a1 and b2/a2, respectively.
     %press a key to continue
s1=freq(b1,a1,n);
s2=freq(b2,a2,n);
%     Next we use these frequency responses to form the theoretical
%     spectral densities.
       %press a key to continue
ss1=se1*s1.*conj(s1);
ss2=se2*s2.*conj(s2);
nn=max(size(ss1));ff=2*pi*(0:nn-1)/nn;
%      Now let's look at the spectral density of the "signal".
     %press a key to see spectral density of the "signal"
%semilogy(ff,ss1),title('spectral density of signal'),  
clc
%     Now let's look at the spectral density of the noise.
      %press a key to see spectral density of the noise.
%semilogy(ff,ss2),title('spectral density of noise'),  
clc
%     Now we'll calculate the two-sided seasonal adjustment filter.
%     We first calculate the Fourier transform of the filter
%     as ss1/(ss1+ss2).   
     %press any key to continue
hf=ss1./(ss1 + ss2);
ha=abs(hf);
nn=max(size(ha));f=2*pi*(0:nn-1)/nn;
      %Now let's plot the amplitude of the filter.  Press a key
plot(f,ha),title('amplitude of two-sided filter'),  
clc
%     Now we'll inverse Fourier transform to obtain the filter.
      %press a key to continue
n2=nn/2;
h=ifft(hf);h=real(h);
hp=[h(n2+1:nn),h(1:n2+1)];
jj=[(-n2:1:0),(1:n2)];
     %Let's plot the filter.  Press a key to see plot
plot(jj,hp),title('two sided filter'),  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  now we'll compute transfer fn for the seas unadjusted series
a=[1 -.9 0 0 -.5 .3];
b=[1 0];
fr=freq(b,a,n); 

lam = .95       %  setting for lambda
ns=n;
llam=[1 -lam]';
llam=[llam;zeros(ns-2,1)];
flam=fft(llam);

fbb=fr';
frat=fbb./conj(flam);
gh=ifft(frat);
hh=real(gh);
no=max(size(hh));
hhh=hh;hhh(no/2+2:no)=zeros(no/2-1,1);
%plot(hhh),title('coeffs on innovs to y for forming geom sum'),pause
fhhh=fft(hhh);
s=abs(fhhh);nn=max(size(s));f=2*pi*(0:nn-1)/nn;
%semilogy(f,s),title('freq response of geom sum'),pause
fr=fr.';
k=ifft(fhhh./fr);
kk=real(k);
knsa=real(k);        % coeficients for the nsa data
kfrnsa=fhhh./fr;		% frequency response for the nsa data
%plot(kk),title('coefficients on y s for forming geom sum'),pause


%semilogy(abs(kfrsa)),title('seasonally adjusted'),pause
%semilogy(abs(kfrnsa)),title('not seasonally adjusted'),pause


%semilogy([abs(kfrnsa) abs(kfrsa)]) ,pause


return


clc
%     Now we'll move on to get the one-sided filter
%     First we have to "factor" ss1 + ss2, which we do using
%     the program "factor".
     %press a key to continue
ss3=(ss1+ss2)';
[b,se]=factor(ss3);
b=[b(1:n2);zeros(n2,1)];
%     Now we'll take the Fourier transform of the moving average
%     kernel B(L), to get B(z).  Then we'll form 
%     Sx(z)/B(z^(-1)) as called for in the filtering formula.
     %press a key to continue
bf=fft(b);
g1=ss1'./(conj(bf)*se);
%     Now we'll inverse Fourier transform g2=Sx(z)/B(z^(-1)) and
%     "annihilate" its negative powers of z, as called for
%     by the optimal filtering formula.
      %press a key to continue
g2=ifft(g1);
g2=[g2(1:n2);zeros(n2,1)];
g2=real(g2);
     %press a key to continue
clc
%     Now Fourier transform the annihilated g2, call it g,
%     divide by B(z), and inverse Fourier transform to get
%     the one-sided filter.
      %press a key to continue
g=fft(g2);
g=g./bf;
h1=ifft(g);
h1=real(h1);
%    Now let's plot the one-sided filter.
      %press a key to see the one-sided filter
plot(h1),title('one-sided filter on noise plus signal'),  
clc
%     Now let's look at the spectral densities of the seasonally
%     adjusted series.
s5=(ss1+ss2).*hf.^2;
s6=(ss1+ss2)'.*(abs(g)).^2;
%     First look at the spectral density of the series
%     that has been adjusted with the optimal two-sided filter.
%     Notice the "dips" in the spectrum at the seasonal frequencies.
      %press a key to get the spectrum
clc
semilogy(ff,s5),title('spectral density of two-sided s.a.'),  
%     Now look at the spectral density of the series that has
%     been adjusted with the optimal one-sided filter.
%     If you used the first-order a.r. specification for the
%     signal, then notice that the spectrum of the adjusted series
%     has no dips at the seasonal frequencies.  Also, notice
%     that the spectrum of the one-sided adjusted series is 
%     proportional to the spectrum of the signal.  (You can
%     prove that this property must hold.)
       %press a key to see the spectrum
clc
semilogy(ff,s6),title('spectral density of one-sided s.a.'),  
%     Now let's plot spectra of both adjusted series on the same
%     graph.
%     The spectrum of the two-sided adjusted series has a lower
%     one-step ahead prediction error variance, which is reflected
%     in the fact that its spectrum is lower.
s7=[s5',s6];
      %press a key to see the spectral densities
semilogy(ff,s7),title('spectra of both adjusted series'),  
clc
%     Now try experimenting with the program by making up
%     your own processes for signals and noises.
%     For example, use the program to construct some estimates
%     of the "seasonal" part of the series, by flipping the roles
%     played by the signal and the noise: i.e., make the signal
%     have a spectral density with peaks at the seasonal frequencies
%     by choosing a1= [0 0 0 .5], for example, and make the noise
%     have no seasonal in it.
%     To do this, just call the fildemo again.
%
%
      %Press a key to proceed with demonstration
clc
s8=[s5',s6,ss3];
     %Press a key to see spectra of original series and both sa
st='spectra of series and one- and two-sided estimator of signal';
semilogy(ff,s8),title(st),  
      %Press a key to return to menu
