
function[lambdah,pihh,ubhat]=can(beta,lambda,pih,deltah,thetah,ub,a22)
% function[lambdah,pihh,ubhat]=can(beta,lambda,pih,deltah,thetah,ub,a22)
%   Computes canonical representation of preferences by
%   solving the auxiliary choice problem
%    maximize  E { Sum beta ^t s(t)'s(t) }
%     subject to
%
%            h(t) = deltah*h(t-1) + thetah*c(t)
%            s(t) = lambda*h(t-1) + pih*c(t)
%    h(-1) given.  The solution of the problem is a feedback rule
%            c(t) = - F*h(t-1),
%    where F = (pih'*pih +beta thetah'*P*thetah)^(-1)
%                        *(beta*thetah'*P*deltah + pih'*lambda),
%    where P solves the algebraic matrix Riccati equation associated
%    with the auxiliary problem.  Then the canonical representation
%    of preferences is given by (pihh,lambdahh) where
%             s(t) = lambdah*h(t-1) + pihh*c(t)
%    and where pihh^(-1)*lambdah = F.
[ns,nh]=size(lambda);
eps=1e-7;
A=deltah;
B=thetah;
Q=lambda'*lambda+eye(nh)*eps;;
R=pih'*pih;
W=lambda'*pih;
[f,p]=olrp(beta,A,B,Q,R,W);
G=pih'*pih + beta*thetah'*p*thetah;
pihh=chol(G);
lambdah=pihh*f;

pihhi=inv(pihh);
S=doublej2(beta*deltah',lambda',a22',ub');
B=pih'*ub + beta*thetah'*S*a22;
D=deltah -thetah*pihhi*lambdah;
SS=doublej2(beta*D',(pihhi*lambdah)',a22',B');
BB=pihhi'*B -beta*pihhi'*thetah'*SS*a22;
ubhat=BB;

