function[pihh,lambdah]=canon(beta,lambda,pih,deltah,thetah)
%function[pihh,lambdah]=canon(beta,lambda,pih,deltah,thetah)
%
%   Computes canonical representation of preferences by
%   solving the auxiliary choice problem
%
%    maximize  E { Sum beta ^t s(t)'s(t) }
%
%     subject to
%
%            h(t) = deltah*h(t-1) + thetah*c(t)
%            s(t) = lambda*h(t-1) + pih*c(t)
%
%    h(-1) given.  The solution of the problem is a feedback rule
%            c(t) = - F*h(t-1),
%
%    where F = (pih'*pih +beta thetah'*P*thetah)^(-1)
%                        *(beta*thetah'*P*deltah + pih'*lambda),
%    where P solves the algebraic matrix Riccati equation associated
%    with the auxiliary problem.  Then the canonical representation
%    of preferences is given by (pihh,lambdahh) where
%
%             s(t) = lambdah*h(t-1) + pihh*c(t)
%    and where pihh^(-1)*lambdah = F.
%
%

A=deltah;
B=thetah;
Q=lambda'*lambda;
R=pih'*pih;
W=lambda'*pih;
[f,p]=olrp(beta,A,B,Q,R,W);
G=pih'*pih + beta*thetah'*p*thetah;
pihh=sqrt(G);
lambdah=pihh*f;
