function[K,S,ST,Gw]=rfilter2(A,G,C,D,H,sig)
%function[K,S,ST,Gw]=rfilter2(A,G,C,D,H,sig)
% robust filter program 
%  K is the robust Kalman gain
%  S is the robust state covariance matrix (for x)
%  Gw is for forming the worst case mean shock v_t = G_w x_t
%
%  Here is the filtering problem;
%
%  x_{t+1} = A x_t +  C w_{t+1} ,  E C C' = Q 
%
%  y_t = G x_t   +   D w{t+1},   E D D'  = R
%
%  z_t = H x_t   ,   H is n_k \times n  
%
%  where z_t is the variable to be estimated.
%  K is for a standard innovations representation
%  Gw is the matrix giving the mean of the worst case shock
%       v_{wt} = G_w x_t
%
%  where G_w = sig*[C - K D]  where 
%  C w_{t+1} is the shock in the state and D w_{t+1} is the
%  shock in the observer.  To match to above take 
%  Q = C C' and R = D D' and C of the form [sqrt Q 0]
%  and D of the form [0 sqrt D].  
%  This problem is dual to the control problem:
%
%  x_{t+1} = A' x_t + G' u_t + H' w_{t+1}
%  
%  with objective function sum_t ( x_t'  Q  x_t + u_t R x_t)
%  We compute using the doubling algorithm.
%   
%  the conditional mean of the worst shock is Gw*x_t
%  

Q = C*C';     
R=D*D';   

[K,S,ST]=doublex4(A,G,Q,R,H,sig);
nq=length(Q); nr=length(R);
DQ=sqrt(Q);
DR=sqrt(K*R*K');
Dp=[DQ; 
   DR];    %  here Dp is D' in the paper 


Gw=sig*((eye(length(Dp))-sig*Dp*S*Dp')\Dp*S*A);

%Gw=sig*(C-K*D)';  