%  SEASLA.M
%
%  Computes time-invariant representation of data generated by
%  periodic model.  SOLVES.M and  SIMULS.M must be run first, and their
%  output must reside in memory.
%
%  This program creates a Wold representation corresponding to a time
%  invariant representation for a single variable, say c(t), that is
%  governed by a strictly periodic model with period p.  The time invariant
%  representation is derived using the state-space methods described in
%  Hansen and Sargent.  The time invariant representation amounts to
%  averaging over p versions of the original process in a way that corresponds
%  to ignoring knowledge of "what quarter we're in".  The program returns
%  the impulse response function of c(t) with respect to the innovation
%  from the Wold representation in z2.  The vector z1 contains the response
%  with respect to the nonfundamental representation.
%
%  Edit the line indicated below to get a series other than c(t).
n=max(size(ao1));
I1=eye(n*p);
AA=zeros(n*p^2);
np=n*p;
for j=1:p-1
eval(['AA(j*np+1:(j+1)*np,(j-1)*np+1:j*np)=I1;'])
end
eval(['AA(1:np,(p-1)*np+1:p*np)=Ah;'])
[m1,m2]=size(Ch);
CC=zeros(n*p^2,m2);
eval(['CC(1:np,1:m2)=Ch;'])
I2=eye(n);
[nc,dumm]=size(sc1);
H=zeros(nc,n*p^2);  %  Note: edit this line and the previous one to get
for j=1:p           %        a series other than c(t).
nj=int2str(j);
%   Note: edit the next line to generate series other than c(t).
eval(['H(:,(j-1)*np+(j-1)*n+1:(j-1)*np+j*n)=si',nj,';'])
end
H=p^(-.5)*H;
										
%  Use Kalman filter to create Wold representation
V1=CC*CC';
V2=eye(nc)*.00000001;
[K,S]=kfilter(AA,H,V1,V2)
										
% Get impulse response function of the state space system to the
% shock hitting agents' information set.
dd=zeros(1,m2);
disp('impulse response w.r.t. innov in agents info (the w`s)  ')
z1=real(dimpulse(AA,CC,H,dd,1,530))
disp('impulse response w.r.t. innov in econometrician`s observation   ')
z2=real(dimpulse(AA,K,H,1,1,530))
