%  testrob.m
%  tests robust filter program for Muth permanent income example
% corrected, June 1, 1998
A=1;
R=1;   %  variance of measurement noise 
G=1;   %  
Q=1;   % variance of state noise
H=1;   %  the variable to be forecast 
sig=0;

[k1,s1,st1,Gw]=rfilter(A,G,Q,R,H,sig);
akg1=A-k1*G 
Gw1=Gw
s1
sig=.2;   %  check the sign convention   
          %  It should be positive when solving the filtering problem

[k2,s2,st2,Gw]=rfilter(A,G,Q,R,H,sig);
akg2=A-k2*G
s2
Gw2=Gw

return

%   Now form two state space systems for the purpose of
%   computing the impulse response function of
%   ac_t w.r.t a_t

ss1=ss(A,k1,G,1,1);   %  the state space system for the first true
		      %  observer
		      %     xh_{t+1} = A xh_t + k1 a_t
		      %       y_t = G xh_t + a_t   


ss2=ss(A-k2*G,k2,-G,1,1);

		      % state space system for the whitener for
                      % the cup system
		      %   
		      %    xc_{t+1} = (A - k2*G) xc_t + k2*y_t
		      %      ac_t   = - G xc_t + y_t





ss3=series(ss1,ss2);  % system with a_t as input, ac_t as output


ss4=ss(A,k2,G,1,1);      %  state space sysem for xc_ with
 		  	 %  ac_t as input
		         %  
			 %  xc_{t+1} = A xc_t + k2 ac_t
		  	 %   y_t = G xc_t + ac_t
		         %




ss5=series(ss3,ss4);     %  series system with a_t as input, y_t as output
			 %  just a check; should agree with results from ss1

%   we will use ss4 for getting impulse response from ac_t to
%		the y_t process from the \check model  


[y3,t3]=impulse(ss3,10);   %  impulse from a_t to ac_t


[y1,t1]=impulse(ss1,10);   %  impulse from a_t to y_t  (true)

[y5,t5]=impulse(ss5,10);   %  impulse from a_t to y_t (true, alternative)

[y4,t4]=impulse(ss4,10);   %  impulse from ac_t to y_t (robust)

%  phony system with zero output to fool the graphs 
%
ssphon=ss(0,0,G,1,1);
[yphone,t1]=impulse(ssphon);


%figure(1) 
%impulse(ss5,10),title('robust')
%print -deps robf1.eps


%figure(2)
%impulse(ss1,10),title('true')
%print -deps robf2.eps

%figure(3)
%impulse(ss4,10),title('impulse with ac as input')
%print -deps robf3.eps

figure(4)
%  This will give the true (blue) and robust (green) impulse responses 
impulse(ssphon,'w.',ss4,'g-',ss1,'b--',10)
% title('blue true, green approx')
gtext('true')
gtext('robust')

print -deps robf1.eps


%  plot the entropy decomposition of the models


%    First, form the two systems for the reconstruction errors
%
%    the standard filter system
%
%    e_{t+1} = (A-k1*G) e_t + D w_{t+1}
%      e_t = 1 * e_t + 0 w_{t+1}
%       D = sqrt(Q +k1*R*k1')
%
D1=sqrt(Q+k1*R*k1'); D2=sqrt(Q+k2*R*k2');

c1=sqrt(Q); c2=sqrt(R);


ss6=ss(akg1,[c1 -k1*c2],1,[0 0],1);
ss7=ss(akg2,[c1 -k2*c2],1,[0 0],1);



covx1=dlyap(akg1,Q+k1*R*k1')   % state covariance from lyapunov for optimal 
covx2=dlyap(akg2,Q+k2*R*k2')   % state covariance from lyapunov for robust


figure(1)
[GD6,ww]=entrop1(ss6);   %  optimal 
[GD7,ww]=entrop1(ss7);   %  robust
figure(6)
%axis([0 pi -inf inf])
plot(ww,GD6,'-b')
hold on
plot(ww,GD7,'--g')
%title('optimal  blue, robust green')
gtext('optimal')
gtext('robust')
hold off
print -deps robf2.eps
figure(5)
impulse(ss3,10)
%title('impulse response from a_t to  ac_t')
print -deps robf3.eps 



figure(8)

%axis([0 pi -inf inf])
subplot(1,2,1)
plot(ww,GD6,'-b')
hold on
plot(ww,GD7,'--g')
%title('optimal  blue, robust green')
gtext('optimal')
gtext('robust')
hold off



%  This will give the true (blue) and robust (green) impulse responses 
subplot(1,2,2)
%impop=impulse(ss1,10);
%imrob=impulse(ss4,10);
%axis([0 10 -.5 1])
%ii=[0:1:10];
%plot(ii,impop,'-b',ii,imrob,'-g')
impulse(ssphon,'w.',ss4,'g-',ss1,'b--',10)
% title('blue true, green approx')
gtext('true')
gtext('robust')


print -deps robf4.eps
