function [AA,V1]=varrep(ao,c,sy,D,R,nj,nnc)
%function [AA,V1]=varrep(ao,c,sy,D,R,nj,nnc)
%   VARREP computes (an infinite order) vector autoregressive representation
%   for a recursive linear model whose observations are corrupted
%   by first-order serially correlated measurement errors.
%   The model occurs in the state space form
%             x(t+1) = ao*x(t) + c*w(t+1)
%               y(t) = sy*x(t) + e(t+1)
%   where e(t) is a measurement error process
%              e(t+1) = D*e(t) + ee(t+1)
%   and where ee(t+1) is a vector white noise with covariance matrix
%   R.  We assume that ee(t+1) and w(t+1) are orthogonal at all
%   leads and lags.  The program computes the autogressive representation
%            y(t) = Sum(j=1, infinity) A(j) y(t-j) + a(t)
%   where a(t)=y(t) - E[y(t)| y(t-1),y(t-2),...], and
%   the A(j) are square matrices.  The program creates
%   the covariance matrix of a, which it stores in V1.
%   The program returns nj  of the matrices A(j), stacked
%   into the ((m times nj) by m) matrix AA, where m is the
%   number of rows of y.  A(j) occurs in rows ((j-1)*m+1) to
%   row j*m of AA.
%   nnc is the location of the constant term in the state vector
% 

   A1=rowd(ao,nnc);
   A=cold(A1,nnc);
   C=rowd(c,nnc);
   G=cold(sy,nnc);
   ao=A;sy=G;c=C;
     
   Q=c*c';
   G=sy;
   [m,n]=size(sy);
   Vm=eye(m);
   Vmm=zeros(m);
   GG=G*ao-D*G;
   Q1=Q;
   W1=Q*G';
   R1=R+G*Q*G';
   [k1,s1]=kfilter(ao,GG,Q1,R1,W1);
   V1=R1+GG*s1*GG';

AA=zeros(nj*m,m);

AA(1:m,:)=D+GG*k1;
for j=2:nj
AA((j-1)*m+1:j*m,:)=GG*(ao-k1*GG)^(j-1)*k1-D*GG*(ao-k1*GG)^(j-2)*k1;
end
AA=AA;
